from dataclasses import dataclass, field

BackendMode = str

PROXY_BACKEND_MODE: BackendMode = "proxy"
WEB_BACKEND_MODE: BackendMode = "web"
TCP_TUNNEL_BACKEND_MODE: BackendMode = "tcpTunnel"
UDP_TUNNEL_BACKEND_MODE: BackendMode = "udpTunnel"
CADDY_BACKEND_MODE: BackendMode = "caddy"

ShareMode = str

PRIVATE_SHARE_MODE: ShareMode = "private"
PUBLIC_SHARE_MODE: ShareMode = "public"


@dataclass
class ShareRequest:
    BackendMode: BackendMode
    ShareMode: ShareMode
    Target: str
    Frontends: list[str] = field(default_factory=list[str])
    BasicAuth: list[str] = field(default_factory=list[str])
    OauthProvider: str = ""
    OauthEmailAddressPatterns: list[str] = field(default_factory=list[str])
    OauthAuthorizationCheckInterval: str = ""
    Reserved: bool = False
    UniqueName: str = ""


@dataclass
class Share:
    Token: str
    FrontendEndpoints: list[str]


@dataclass
class AccessRequest:
    ShareToken: str


@dataclass
class Access:
    Token: str
    ShareToken: str
    BackendMode: BackendMode


@dataclass
class SessionMetrics:
    BytesRead: int
    BytesWritten: int
    LastUpdate: int


@dataclass
class Metrics:
    Namespace: str
    Sessions: dict[str, SessionMetrics]


AuthScheme = str

AUTH_SCHEME_NONE: AuthScheme = "none"
AUTH_SCHEME_BASIC: AuthScheme = "basic"
AUTH_SCHEME_OAUTH: AuthScheme = "oauth"



# coding: utf-8

"""
    zrok

    zrok client access  # noqa: E501

    OpenAPI spec version: 0.3.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ShareRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'env_zid': 'str',
        'share_mode': 'str',
        'frontend_selection': 'list[str]',
        'backend_mode': 'str',
        'backend_proxy_endpoint': 'str',
        'auth_scheme': 'str',
        'auth_users': 'list[AuthUser]',
        'oauth_provider': 'str',
        'oauth_email_domains': 'list[str]',
        'oauth_authorization_check_interval': 'str',
        'reserved': 'bool',
        'unique_name': 'str'
    }

    attribute_map = {
        'env_zid': 'envZId',
        'share_mode': 'shareMode',
        'frontend_selection': 'frontendSelection',
        'backend_mode': 'backendMode',
        'backend_proxy_endpoint': 'backendProxyEndpoint',
        'auth_scheme': 'authScheme',
        'auth_users': 'authUsers',
        'oauth_provider': 'oauthProvider',
        'oauth_email_domains': 'oauthEmailDomains',
        'oauth_authorization_check_interval': 'oauthAuthorizationCheckInterval',
        'reserved': 'reserved',
        'unique_name': 'uniqueName'
    }

    def __init__(self, env_zid=None, share_mode=None, frontend_selection=None, backend_mode=None, backend_proxy_endpoint=None, auth_scheme=None, auth_users=None, oauth_provider=None, oauth_email_domains=None, oauth_authorization_check_interval=None, reserved=None, unique_name=None):  # noqa: E501
        """ShareRequest - a model defined in Swagger"""  # noqa: E501
        self._env_zid = None
        self._share_mode = None
        self._frontend_selection = None
        self._backend_mode = None
        self._backend_proxy_endpoint = None
        self._auth_scheme = None
        self._auth_users = None
        self._oauth_provider = None
        self._oauth_email_domains = None
        self._oauth_authorization_check_interval = None
        self._reserved = None
        self._unique_name = None
        self.discriminator = None
        if env_zid is not None:
            self.env_zid = env_zid
        if share_mode is not None:
            self.share_mode = share_mode
        if frontend_selection is not None:
            self.frontend_selection = frontend_selection
        if backend_mode is not None:
            self.backend_mode = backend_mode
        if backend_proxy_endpoint is not None:
            self.backend_proxy_endpoint = backend_proxy_endpoint
        if auth_scheme is not None:
            self.auth_scheme = auth_scheme
        if auth_users is not None:
            self.auth_users = auth_users
        if oauth_provider is not None:
            self.oauth_provider = oauth_provider
        if oauth_email_domains is not None:
            self.oauth_email_domains = oauth_email_domains
        if oauth_authorization_check_interval is not None:
            self.oauth_authorization_check_interval = oauth_authorization_check_interval
        if reserved is not None:
            self.reserved = reserved
        if unique_name is not None:
            self.unique_name = unique_name

    @property
    def env_zid(self):
        """Gets the env_zid of this ShareRequest.  # noqa: E501


        :return: The env_zid of this ShareRequest.  # noqa: E501
        :rtype: str
        """
        return self._env_zid

    @env_zid.setter
    def env_zid(self, env_zid):
        """Sets the env_zid of this ShareRequest.


        :param env_zid: The env_zid of this ShareRequest.  # noqa: E501
        :type: str
        """

        self._env_zid = env_zid

    @property
    def share_mode(self):
        """Gets the share_mode of this ShareRequest.  # noqa: E501


        :return: The share_mode of this ShareRequest.  # noqa: E501
        :rtype: str
        """
        return self._share_mode

    @share_mode.setter
    def share_mode(self, share_mode):
        """Sets the share_mode of this ShareRequest.


        :param share_mode: The share_mode of this ShareRequest.  # noqa: E501
        :type: str
        """
        allowed_values = ["public", "private"]  # noqa: E501
        if share_mode not in allowed_values:
            raise ValueError(
                "Invalid value for `share_mode` ({0}), must be one of {1}"  # noqa: E501
                .format(share_mode, allowed_values)
            )

        self._share_mode = share_mode

    @property
    def frontend_selection(self):
        """Gets the frontend_selection of this ShareRequest.  # noqa: E501


        :return: The frontend_selection of this ShareRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._frontend_selection

    @frontend_selection.setter
    def frontend_selection(self, frontend_selection):
        """Sets the frontend_selection of this ShareRequest.


        :param frontend_selection: The frontend_selection of this ShareRequest.  # noqa: E501
        :type: list[str]
        """

        self._frontend_selection = frontend_selection

    @property
    def backend_mode(self):
        """Gets the backend_mode of this ShareRequest.  # noqa: E501


        :return: The backend_mode of this ShareRequest.  # noqa: E501
        :rtype: str
        """
        return self._backend_mode

    @backend_mode.setter
    def backend_mode(self, backend_mode):
        """Sets the backend_mode of this ShareRequest.


        :param backend_mode: The backend_mode of this ShareRequest.  # noqa: E501
        :type: str
        """
        allowed_values = ["proxy", "web", "tcpTunnel", "udpTunnel", "caddy", "drive", "socks"]  # noqa: E501
        if backend_mode not in allowed_values:
            raise ValueError(
                "Invalid value for `backend_mode` ({0}), must be one of {1}"  # noqa: E501
                .format(backend_mode, allowed_values)
            )

        self._backend_mode = backend_mode

    @property
    def backend_proxy_endpoint(self):
        """Gets the backend_proxy_endpoint of this ShareRequest.  # noqa: E501


        :return: The backend_proxy_endpoint of this ShareRequest.  # noqa: E501
        :rtype: str
        """
        return self._backend_proxy_endpoint

    @backend_proxy_endpoint.setter
    def backend_proxy_endpoint(self, backend_proxy_endpoint):
        """Sets the backend_proxy_endpoint of this ShareRequest.


        :param backend_proxy_endpoint: The backend_proxy_endpoint of this ShareRequest.  # noqa: E501
        :type: str
        """

        self._backend_proxy_endpoint = backend_proxy_endpoint

    @property
    def auth_scheme(self):
        """Gets the auth_scheme of this ShareRequest.  # noqa: E501


        :return: The auth_scheme of this ShareRequest.  # noqa: E501
        :rtype: str
        """
        return self._auth_scheme

    @auth_scheme.setter
    def auth_scheme(self, auth_scheme):
        """Sets the auth_scheme of this ShareRequest.


        :param auth_scheme: The auth_scheme of this ShareRequest.  # noqa: E501
        :type: str
        """

        self._auth_scheme = auth_scheme

    @property
    def auth_users(self):
        """Gets the auth_users of this ShareRequest.  # noqa: E501


        :return: The auth_users of this ShareRequest.  # noqa: E501
        :rtype: list[AuthUser]
        """
        return self._auth_users

    @auth_users.setter
    def auth_users(self, auth_users):
        """Sets the auth_users of this ShareRequest.


        :param auth_users: The auth_users of this ShareRequest.  # noqa: E501
        :type: list[AuthUser]
        """

        self._auth_users = auth_users

    @property
    def oauth_provider(self):
        """Gets the oauth_provider of this ShareRequest.  # noqa: E501


        :return: The oauth_provider of this ShareRequest.  # noqa: E501
        :rtype: str
        """
        return self._oauth_provider

    @oauth_provider.setter
    def oauth_provider(self, oauth_provider):
        """Sets the oauth_provider of this ShareRequest.


        :param oauth_provider: The oauth_provider of this ShareRequest.  # noqa: E501
        :type: str
        """
        allowed_values = ["github", "google"]  # noqa: E501
        if oauth_provider not in allowed_values:
            raise ValueError(
                "Invalid value for `oauth_provider` ({0}), must be one of {1}"  # noqa: E501
                .format(oauth_provider, allowed_values)
            )

        self._oauth_provider = oauth_provider

    @property
    def oauth_email_domains(self):
        """Gets the oauth_email_domains of this ShareRequest.  # noqa: E501


        :return: The oauth_email_domains of this ShareRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._oauth_email_domains

    @oauth_email_domains.setter
    def oauth_email_domains(self, oauth_email_domains):
        """Sets the oauth_email_domains of this ShareRequest.


        :param oauth_email_domains: The oauth_email_domains of this ShareRequest.  # noqa: E501
        :type: list[str]
        """

        self._oauth_email_domains = oauth_email_domains

    @property
    def oauth_authorization_check_interval(self):
        """Gets the oauth_authorization_check_interval of this ShareRequest.  # noqa: E501


        :return: The oauth_authorization_check_interval of this ShareRequest.  # noqa: E501
        :rtype: str
        """
        return self._oauth_authorization_check_interval

    @oauth_authorization_check_interval.setter
    def oauth_authorization_check_interval(self, oauth_authorization_check_interval):
        """Sets the oauth_authorization_check_interval of this ShareRequest.


        :param oauth_authorization_check_interval: The oauth_authorization_check_interval of this ShareRequest.  # noqa: E501
        :type: str
        """

        self._oauth_authorization_check_interval = oauth_authorization_check_interval

    @property
    def reserved(self):
        """Gets the reserved of this ShareRequest.  # noqa: E501


        :return: The reserved of this ShareRequest.  # noqa: E501
        :rtype: bool
        """
        return self._reserved

    @reserved.setter
    def reserved(self, reserved):
        """Sets the reserved of this ShareRequest.


        :param reserved: The reserved of this ShareRequest.  # noqa: E501
        :type: bool
        """

        self._reserved = reserved

    @property
    def unique_name(self):
        """Gets the unique_name of this ShareRequest.  # noqa: E501


        :return: The unique_name of this ShareRequest.  # noqa: E501
        :rtype: str
        """
        return self._unique_name

    @unique_name.setter
    def unique_name(self, unique_name):
        """Sets the unique_name of this ShareRequest.


        :param unique_name: The unique_name of this ShareRequest.  # noqa: E501
        :type: str
        """

        self._unique_name = unique_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ShareRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ShareRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

"""
    zrok

    zrok client access  # noqa: E501

    OpenAPI spec version: 0.3.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import apis into sdk package
from zrok_api.api.account_api import AccountApi
from zrok_api.api.admin_api import AdminApi
from zrok_api.api.environment_api import EnvironmentApi
from zrok_api.api.metadata_api import MetadataApi
from zrok_api.api.share_api import ShareApi
# import ApiClient
from zrok_api.api_client import ApiClient
from zrok_api.configuration import Configuration
# import models into sdk package
from zrok_api.models.access_request import AccessRequest
from zrok_api.models.access_response import AccessResponse
from zrok_api.models.auth_user import AuthUser
from zrok_api.models.change_password_request import ChangePasswordRequest
from zrok_api.models.configuration import Configuration
from zrok_api.models.create_frontend_request import CreateFrontendRequest
from zrok_api.models.create_frontend_response import CreateFrontendResponse
from zrok_api.models.delete_frontend_request import DeleteFrontendRequest
from zrok_api.models.disable_request import DisableRequest
from zrok_api.models.enable_request import EnableRequest
from zrok_api.models.enable_response import EnableResponse
from zrok_api.models.environment import Environment
from zrok_api.models.environment_and_resources import EnvironmentAndResources
from zrok_api.models.environments import Environments
from zrok_api.models.error_message import ErrorMessage
from zrok_api.models.frontend import Frontend
from zrok_api.models.frontends import Frontends
from zrok_api.models.identity_body import IdentityBody
from zrok_api.models.inline_response200 import InlineResponse200
from zrok_api.models.inline_response201 import InlineResponse201
from zrok_api.models.invite_request import InviteRequest
from zrok_api.models.invite_token_generate_request import InviteTokenGenerateRequest
from zrok_api.models.login_request import LoginRequest
from zrok_api.models.login_response import LoginResponse
from zrok_api.models.metrics import Metrics
from zrok_api.models.metrics_sample import MetricsSample
from zrok_api.models.overview import Overview
from zrok_api.models.password_requirements import PasswordRequirements
from zrok_api.models.principal import Principal
from zrok_api.models.public_frontend import PublicFrontend
from zrok_api.models.public_frontend_list import PublicFrontendList
from zrok_api.models.regenerate_token_body import RegenerateTokenBody
from zrok_api.models.register_request import RegisterRequest
from zrok_api.models.register_response import RegisterResponse
from zrok_api.models.reset_password_request import ResetPasswordRequest
from zrok_api.models.reset_password_request_body import ResetPasswordRequestBody
from zrok_api.models.share import Share
from zrok_api.models.share_request import ShareRequest
from zrok_api.models.share_response import ShareResponse
from zrok_api.models.shares import Shares
from zrok_api.models.spark_data import SparkData
from zrok_api.models.spark_data_sample import SparkDataSample
from zrok_api.models.unaccess_request import UnaccessRequest
from zrok_api.models.unshare_request import UnshareRequest
from zrok_api.models.update_frontend_request import UpdateFrontendRequest
from zrok_api.models.update_share_request import UpdateShareRequest
from zrok_api.models.verify_request import VerifyRequest
from zrok_api.models.verify_response import VerifyResponse
from zrok_api.models.version import Version



import openziti
from zrok.environment.root import Root


class Listener():
    shrToken: str
    root: Root
    __server: openziti.zitisock.ZitiSocket

    def __init__(self, shrToken: str, root: Root):
        self.shrToken = shrToken
        self.root = root
        ztx = openziti.load(
            self.root.ZitiIdentityNamed(
                self.root.EnvironmentIdentityName()))
        self.__server = ztx.bind(self.shrToken)

    def __enter__(self) -> openziti.zitisock.ZitiSocket:
        self.listen()
        return self.__server

    def __exit__(self, exception_type, exception_value, exception_traceback):
        self.close()

    def listen(self):
        self.__server.listen()

    def close(self):
        self.__server.close()



# coding: utf-8

"""
    zrok

    zrok client access  # noqa: E501

    OpenAPI spec version: 0.3.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class PasswordRequirements(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'length': 'int',
        'require_capital': 'bool',
        'require_numeric': 'bool',
        'require_special': 'bool',
        'valid_special_characters': 'str'
    }

    attribute_map = {
        'length': 'length',
        'require_capital': 'requireCapital',
        'require_numeric': 'requireNumeric',
        'require_special': 'requireSpecial',
        'valid_special_characters': 'validSpecialCharacters'
    }

    def __init__(self, length=None, require_capital=None, require_numeric=None, require_special=None, valid_special_characters=None):  # noqa: E501
        """PasswordRequirements - a model defined in Swagger"""  # noqa: E501
        self._length = None
        self._require_capital = None
        self._require_numeric = None
        self._require_special = None
        self._valid_special_characters = None
        self.discriminator = None
        if length is not None:
            self.length = length
        if require_capital is not None:
            self.require_capital = require_capital
        if require_numeric is not None:
            self.require_numeric = require_numeric
        if require_special is not None:
            self.require_special = require_special
        if valid_special_characters is not None:
            self.valid_special_characters = valid_special_characters

    @property
    def length(self):
        """Gets the length of this PasswordRequirements.  # noqa: E501


        :return: The length of this PasswordRequirements.  # noqa: E501
        :rtype: int
        """
        return self._length

    @length.setter
    def length(self, length):
        """Sets the length of this PasswordRequirements.


        :param length: The length of this PasswordRequirements.  # noqa: E501
        :type: int
        """

        self._length = length

    @property
    def require_capital(self):
        """Gets the require_capital of this PasswordRequirements.  # noqa: E501


        :return: The require_capital of this PasswordRequirements.  # noqa: E501
        :rtype: bool
        """
        return self._require_capital

    @require_capital.setter
    def require_capital(self, require_capital):
        """Sets the require_capital of this PasswordRequirements.


        :param require_capital: The require_capital of this PasswordRequirements.  # noqa: E501
        :type: bool
        """

        self._require_capital = require_capital

    @property
    def require_numeric(self):
        """Gets the require_numeric of this PasswordRequirements.  # noqa: E501


        :return: The require_numeric of this PasswordRequirements.  # noqa: E501
        :rtype: bool
        """
        return self._require_numeric

    @require_numeric.setter
    def require_numeric(self, require_numeric):
        """Sets the require_numeric of this PasswordRequirements.


        :param require_numeric: The require_numeric of this PasswordRequirements.  # noqa: E501
        :type: bool
        """

        self._require_numeric = require_numeric

    @property
    def require_special(self):
        """Gets the require_special of this PasswordRequirements.  # noqa: E501


        :return: The require_special of this PasswordRequirements.  # noqa: E501
        :rtype: bool
        """
        return self._require_special

    @require_special.setter
    def require_special(self, require_special):
        """Sets the require_special of this PasswordRequirements.


        :param require_special: The require_special of this PasswordRequirements.  # noqa: E501
        :type: bool
        """

        self._require_special = require_special

    @property
    def valid_special_characters(self):
        """Gets the valid_special_characters of this PasswordRequirements.  # noqa: E501


        :return: The valid_special_characters of this PasswordRequirements.  # noqa: E501
        :rtype: str
        """
        return self._valid_special_characters

    @valid_special_characters.setter
    def valid_special_characters(self, valid_special_characters):
        """Sets the valid_special_characters of this PasswordRequirements.


        :param valid_special_characters: The valid_special_characters of this PasswordRequirements.  # noqa: E501
        :type: str
        """

        self._valid_special_characters = valid_special_characters

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(PasswordRequirements, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PasswordRequirements):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    zrok

    zrok client access  # noqa: E501

    OpenAPI spec version: 0.3.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Principal(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'int',
        'email': 'str',
        'token': 'str',
        'limitless': 'bool',
        'admin': 'bool'
    }

    attribute_map = {
        'id': 'id',
        'email': 'email',
        'token': 'token',
        'limitless': 'limitless',
        'admin': 'admin'
    }

    def __init__(self, id=None, email=None, token=None, limitless=None, admin=None):  # noqa: E501
        """Principal - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._email = None
        self._token = None
        self._limitless = None
        self._admin = None
        self.discriminator = None
        if id is not None:
            self.id = id
        if email is not None:
            self.email = email
        if token is not None:
            self.token = token
        if limitless is not None:
            self.limitless = limitless
        if admin is not None:
            self.admin = admin

    @property
    def id(self):
        """Gets the id of this Principal.  # noqa: E501


        :return: The id of this Principal.  # noqa: E501
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Principal.


        :param id: The id of this Principal.  # noqa: E501
        :type: int
        """

        self._id = id

    @property
    def email(self):
        """Gets the email of this Principal.  # noqa: E501


        :return: The email of this Principal.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this Principal.


        :param email: The email of this Principal.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def token(self):
        """Gets the token of this Principal.  # noqa: E501


        :return: The token of this Principal.  # noqa: E501
        :rtype: str
        """
        return self._token

    @token.setter
    def token(self, token):
        """Sets the token of this Principal.


        :param token: The token of this Principal.  # noqa: E501
        :type: str
        """

        self._token = token

    @property
    def limitless(self):
        """Gets the limitless of this Principal.  # noqa: E501


        :return: The limitless of this Principal.  # noqa: E501
        :rtype: bool
        """
        return self._limitless

    @limitless.setter
    def limitless(self, limitless):
        """Sets the limitless of this Principal.


        :param limitless: The limitless of this Principal.  # noqa: E501
        :type: bool
        """

        self._limitless = limitless

    @property
    def admin(self):
        """Gets the admin of this Principal.  # noqa: E501


        :return: The admin of this Principal.  # noqa: E501
        :rtype: bool
        """
        return self._admin

    @admin.setter
    def admin(self, admin):
        """Sets the admin of this Principal.


        :param admin: The admin of this Principal.  # noqa: E501
        :type: bool
        """

        self._admin = admin

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Principal, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Principal):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from dataclasses import dataclass
import openziti
from zrok.environment.root import Root


@dataclass
class Opts:
    root: Root
    shrToken: str
    bindPort: int
    bindHost: str = ""


class MonkeyPatch(openziti.monkeypatch):
    def __init__(self, opts: {}, *args, **kwargs):
        zif = opts['cfg'].root.ZitiIdentityNamed(opts['cfg'].root.EnvironmentIdentityName())
        cfg = dict(ztx=openziti.load(zif), service=opts['cfg'].shrToken)
        super(MonkeyPatch, self).__init__(bindings={(opts['cfg'].bindHost, opts['cfg'].bindPort): cfg}, *args, **kwargs)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        super(MonkeyPatch, self).__exit__(exc_type, exc_val, exc_tb)


def zrok(opts: {}, *zargs, **zkwargs):
    def zrockify_func(func):
        def zrockified(*args, **kwargs):
            with MonkeyPatch(opts=opts, *zargs, **zkwargs):
                func(*args, **kwargs)
        return zrockified
    return zrockify_func



# coding: utf-8

"""
    zrok

    zrok client access  # noqa: E501

    OpenAPI spec version: 0.3.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Environment(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'host': 'str',
        'address': 'str',
        'z_id': 'str',
        'activity': 'SparkData',
        'limited': 'bool',
        'created_at': 'int',
        'updated_at': 'int'
    }

    attribute_map = {
        'description': 'description',
        'host': 'host',
        'address': 'address',
        'z_id': 'zId',
        'activity': 'activity',
        'limited': 'limited',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt'
    }

    def __init__(self, description=None, host=None, address=None, z_id=None, activity=None, limited=None, created_at=None, updated_at=None):  # noqa: E501
        """Environment - a model defined in Swagger"""  # noqa: E501
        self._description = None
        self._host = None
        self._address = None
        self._z_id = None
        self._activity = None
        self._limited = None
        self._created_at = None
        self._updated_at = None
        self.discriminator = None
        if description is not None:
            self.description = description
        if host is not None:
            self.host = host
        if address is not None:
            self.address = address
        if z_id is not None:
            self.z_id = z_id
        if activity is not None:
            self.activity = activity
        if limited is not None:
            self.limited = limited
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at

    @property
    def description(self):
        """Gets the description of this Environment.  # noqa: E501


        :return: The description of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Environment.


        :param description: The description of this Environment.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def host(self):
        """Gets the host of this Environment.  # noqa: E501


        :return: The host of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._host

    @host.setter
    def host(self, host):
        """Sets the host of this Environment.


        :param host: The host of this Environment.  # noqa: E501
        :type: str
        """

        self._host = host

    @property
    def address(self):
        """Gets the address of this Environment.  # noqa: E501


        :return: The address of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._address

    @address.setter
    def address(self, address):
        """Sets the address of this Environment.


        :param address: The address of this Environment.  # noqa: E501
        :type: str
        """

        self._address = address

    @property
    def z_id(self):
        """Gets the z_id of this Environment.  # noqa: E501


        :return: The z_id of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._z_id

    @z_id.setter
    def z_id(self, z_id):
        """Sets the z_id of this Environment.


        :param z_id: The z_id of this Environment.  # noqa: E501
        :type: str
        """

        self._z_id = z_id

    @property
    def activity(self):
        """Gets the activity of this Environment.  # noqa: E501


        :return: The activity of this Environment.  # noqa: E501
        :rtype: SparkData
        """
        return self._activity

    @activity.setter
    def activity(self, activity):
        """Sets the activity of this Environment.


        :param activity: The activity of this Environment.  # noqa: E501
        :type: SparkData
        """

        self._activity = activity

    @property
    def limited(self):
        """Gets the limited of this Environment.  # noqa: E501


        :return: The limited of this Environment.  # noqa: E501
        :rtype: bool
        """
        return self._limited

    @limited.setter
    def limited(self, limited):
        """Sets the limited of this Environment.


        :param limited: The limited of this Environment.  # noqa: E501
        :type: bool
        """

        self._limited = limited

    @property
    def created_at(self):
        """Gets the created_at of this Environment.  # noqa: E501


        :return: The created_at of this Environment.  # noqa: E501
        :rtype: int
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Environment.


        :param created_at: The created_at of this Environment.  # noqa: E501
        :type: int
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this Environment.  # noqa: E501


        :return: The updated_at of this Environment.  # noqa: E501
        :rtype: int
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Environment.


        :param updated_at: The updated_at of this Environment.  # noqa: E501
        :type: int
        """

        self._updated_at = updated_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Environment, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Environment):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from setuptools import setup, find_packages  # noqa: H301
import os
import versioneer

# optionally upload to TestPyPi with alternative name in testing repo
NAME = os.getenv('ZROK_PY_NAME', "zrok")
VERSION = "1.0.0"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["urllib3 >= 1.15", "six >= 1.10", "certifi", "python-dateutil", "openziti >= 0.8.1"]

setup(
    name=NAME,
    cmdclass=versioneer.get_cmdclass(dict()),
    version=versioneer.get_version(),
    description="zrok",
    author_email="",
    url="",
    keywords=["Swagger", "zrok"],
    install_requires=REQUIRES,
    python_requires='>3.10.0',
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
    Geo-scale, next-generation peer-to-peer sharing platform built on top of OpenZiti.
    """
)



from zrok.environment.root import Root
from zrok_api.models import AccessRequest, UnaccessRequest
from zrok_api.api import ShareApi
from zrok import model


class Access():
    root: Root
    request: model.AccessRequest
    access: model.Access

    def __init__(self, root: Root, request: model.AccessRequest):
        self.root = root
        self.request = request

    def __enter__(self) -> model.Access:
        self.access = CreateAccess(root=self.root, request=self.request)
        return self.access

    def __exit__(self, exception_type, exception_value, exception_traceback):
        DeleteAccess(root=self.root, acc=self.access)


def CreateAccess(root: Root, request: model.AccessRequest) -> model.Access:
    if not root.IsEnabled():
        raise Exception("environment is not enabled; enable with 'zrok enable' first!")

    out = AccessRequest(shr_token=request.ShareToken,
                        env_zid=root.env.ZitiIdentity)

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)
    try:
        res = ShareApi(zrok).access(body=out)
    except Exception as e:
        raise Exception("unable to create access", e)
    return model.Access(Token=res.frontend_token,
                        ShareToken=request.ShareToken,
                        BackendMode=res.backend_mode)


def DeleteAccess(root: Root, acc: model.Access):
    req = UnaccessRequest(frontend_token=acc.Token,
                          shr_token=acc.ShareToken,
                          env_zid=root.env.ZitiIdentity)

    try:
        zrok = root.Client()
    except Exception as e:
        raise Exception("error getting zrok client", e)

    try:
        ShareApi(zrok).unaccess(body=req)
    except Exception as e:
        raise Exception("error deleting access", e)



from dataclasses import dataclass, field
from typing import NamedTuple
from .dirs import identityFile, rootDir, configFile, environmentFile, metadataFile
import os
import json
import zrok_api as zrok
from zrok_api.configuration import Configuration
import re

V = "v0.4"


@dataclass
class Metadata:
    V: str = ""
    RootPath: str = ""


@dataclass
class Config:
    ApiEndpoint: str = ""


@dataclass
class Environment:
    Token: str = ""
    ZitiIdentity: str = ""
    ApiEndpoint: str = ""


class ApiEndpoint(NamedTuple):
    endpoint: str
    frm: str


@dataclass
class Root:
    meta: Metadata = field(default_factory=Metadata)
    cfg: Config = field(default_factory=Config)
    env: Environment = field(default_factory=Environment)

    def HasConfig(self) -> bool:
        return self.cfg != Config()

    def Client(self) -> zrok.ApiClient:
        apiEndpoint = self.ApiEndpoint()

        cfg = Configuration()
        cfg.host = apiEndpoint[0] + "/api/v1"
        cfg.api_key["x-token"] = self.env.Token
        cfg.api_key_prefix['Authorization'] = 'Bearer'

        zrock_client = zrok.ApiClient(configuration=cfg)
        v = zrok.MetadataApi(zrock_client).version()
        # allow reported version string to be optionally prefixed with
        # "refs/heads/" or "refs/tags/"
        rxp = re.compile("^(refs/(heads|tags)/)?" + V)
        if not rxp.match(v):
            raise Exception("expected a '" + V + "' version, received: '" + v + "'")
        return zrock_client

    def ApiEndpoint(self) -> ApiEndpoint:
        apiEndpoint = "https://api.zrok.io"
        frm = "binary"

        if self.cfg.ApiEndpoint != "":
            apiEndpoint = self.cfg.ApiEndpoint
            frm = "config"

        env = os.getenv("ZROK_API_ENDPOINT")
        if env != "":
            apiEndpoint = env
            frm = "ZROK_API_ENDPOINT"

        if self.IsEnabled():
            apiEndpoint = self.env.ApiEndpoint
            frm = "env"

        return ApiEndpoint(apiEndpoint.rstrip("/"), frm)

    def IsEnabled(self) -> bool:
        return self.env != Environment()

    def PublicIdentityName(self) -> str:
        return "public"

    def EnvironmentIdentityName(self) -> str:
        return "environment"

    def ZitiIdentityNamed(self, name: str) -> str:
        return identityFile(name)


def Default() -> Root:
    r = Root()
    root = rootDir()
    r.meta = Metadata(V=V, RootPath=root)
    return r


def Assert() -> bool:
    exists = __rootExists()
    if exists:
        meta = __loadMetadata()
        return meta.V == V
    return False


def Load() -> Root:
    r = Root()
    if __rootExists():
        r.meta = __loadMetadata()
        r.cfg = __loadConfig()
        r.env = __loadEnvironment()
    else:
        r = Default()
    return r


def __rootExists() -> bool:
    mf = metadataFile()
    return os.path.isfile(mf)


def __assertMetadata():
    pass


def __loadMetadata() -> Metadata:
    mf = metadataFile()
    with open(mf) as f:
        data = json.load(f)
        return Metadata(V=data["v"])


def __loadConfig() -> Config:
    cf = configFile()
    with open(cf) as f:
        data = json.load(f)
        return Config(ApiEndpoint=data["api_endpoint"])


def isEnabled() -> bool:
    ef = environmentFile()
    return os.path.isfile(ef)


def __loadEnvironment() -> Environment:
    ef = environmentFile()
    with open(ef) as f:
        data = json.load(f)
        return Environment(
            Token=data["zrok_token"],
            ZitiIdentity=data["ziti_identity"],
            ApiEndpoint=data["api_endpoint"])
